@@ -1021,6 +1021,7 @@ gx_default_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
         case gxdso_supports_hlcolor:
         case gxdso_supports_saved_pages:
         case gxdso_needs_invariant_palette:
+        case gxdso_supports_iccpostrender:
             return 0;
         case gxdso_pattern_shfill_doesnt_need_path:
             return (dev_proc(pdev, fill_path) == gx_default_fill_path);
@@ -1330,7 +1331,7 @@ int gx_device_subclass(gx_device *dev_to_subclass, gx_device *new_prototype, uns
 {
     gx_device *child_dev;
     void *psubclass_data;
-    gs_memory_struct_type_t *a_std;
+    gs_memory_struct_type_t *a_std, *b_std;
     int dynamic = dev_to_subclass->stype_is_dynamic;
     char *ptr, *ptr1;
 
@@ -1354,11 +1355,20 @@ int gx_device_subclass(gx_device *dev_to_subclass, gx_device *new_prototype, uns
     *a_std = *dev_to_subclass->stype;
     a_std->ssize = dev_to_subclass->params_size;
 
+    if (!dynamic) {
+        b_std = (gs_memory_struct_type_t *)
+            gs_alloc_bytes_immovable(dev_to_subclass->memory->non_gc_memory, sizeof(*b_std),
+                                     "gs_device_subclass(stype)");
+        if (!b_std)
+            return_error(gs_error_VMerror);
+    }
+
     /* Allocate a device structure for the new child device */
-    child_dev = gs_alloc_struct_immovable(dev_to_subclass->memory, gx_device, a_std,
+    child_dev = gs_alloc_struct_immovable(dev_to_subclass->memory->stable_memory, gx_device, a_std,
                                         "gs_device_subclass(device)");
     if (child_dev == 0) {
         gs_free_const_object(dev_to_subclass->memory->non_gc_memory, a_std, "gs_device_subclass(stype)");
+        gs_free_const_object(dev_to_subclass->memory->non_gc_memory, b_std, "gs_device_subclass(stype)");
         return_error(gs_error_VMerror);
     }
 
@@ -1374,7 +1384,8 @@ int gx_device_subclass(gx_device *dev_to_subclass, gx_device *new_prototype, uns
     psubclass_data = (void *)gs_alloc_bytes(dev_to_subclass->memory->non_gc_memory, private_data_size, "subclass memory for subclassing device");
     if (psubclass_data == 0){
         gs_free_const_object(dev_to_subclass->memory->non_gc_memory, a_std, "gs_device_subclass(stype)");
-        gs_free(dev_to_subclass->memory, child_dev, 1, dev_to_subclass->stype->ssize, "free subclass memory for subclassing device");
+        gs_free_const_object(dev_to_subclass->memory->non_gc_memory, b_std, "gs_device_subclass(stype)");
+        gs_free_object(dev_to_subclass->memory->stable_memory, child_dev, "free subclass memory for subclassing device");
         return_error(gs_error_VMerror);
     }
     memset(psubclass_data, 0x00, private_data_size);
@@ -1396,26 +1407,34 @@ int gx_device_subclass(gx_device *dev_to_subclass, gx_device *new_prototype, uns
     ptr1 = ((char *)new_prototype) + sizeof(gx_device);
     memcpy(ptr, ptr1, new_prototype->params_size - sizeof(gx_device));
 
-    /* We have to patch up the "type" parameters that the memory manage/garbage
-     * collector will use, as well.
-     */
-    gs_set_object_type(child_dev->memory, dev_to_subclass, new_prototype->stype);
-
     /* If the original device's stype structure was dynamically allocated, we need
      * to 'fixup' the contents, it's procs need to point to the new device's procs
      * for instance.
      */
     if (dynamic) {
         if (new_prototype->stype) {
-            a_std = (gs_memory_struct_type_t *)dev_to_subclass->stype;
-            *a_std = *new_prototype->stype;
+            b_std = (gs_memory_struct_type_t *)dev_to_subclass->stype;
+            *b_std = *new_prototype->stype;
+            b_std->ssize = a_std->ssize;
+            dev_to_subclass->stype_is_dynamic = 1;
         } else {
             gs_free_const_object(child_dev->memory->non_gc_memory, dev_to_subclass->stype,
                              "unsubclass");
             dev_to_subclass->stype = NULL;
+            b_std = (gs_memory_struct_type_t *)new_prototype->stype;
             dev_to_subclass->stype_is_dynamic = 0;
         }
     }
+    else {
+        *b_std = *new_prototype->stype;
+        b_std->ssize = a_std->ssize;
+        dev_to_subclass->stype_is_dynamic = 1;
+    }
+    dev_to_subclass->stype = b_std;
+    /* We have to patch up the "type" parameters that the memory manage/garbage
+     * collector will use, as well.
+     */
+    gs_set_object_type(child_dev->memory, dev_to_subclass, b_std);
 
     dev_to_subclass->subclass_data = psubclass_data;
     dev_to_subclass->child = child_dev;
@@ -1423,6 +1442,9 @@ int gx_device_subclass(gx_device *dev_to_subclass, gx_device *new_prototype, uns
         dev_to_subclass->parent = child_dev->parent;
         child_dev->parent->child = dev_to_subclass;
     }
+    if (child_dev->child) {
+        child_dev->child->parent = child_dev;
+    }
     child_dev->parent = dev_to_subclass;
 
     return 0;
@@ -1430,9 +1452,9 @@ int gx_device_subclass(gx_device *dev_to_subclass, gx_device *new_prototype, uns
 
 int gx_device_unsubclass(gx_device *dev)
 {
-    void *psubclass_data;
+    generic_subclass_data *psubclass_data;
     gx_device *parent, *child;
-    gs_memory_struct_type_t *a_std = 0;
+    gs_memory_struct_type_t *a_std = 0, *b_std = 0;
     int dynamic, ref_count;
 
     /* This should not happen... */
@@ -1441,10 +1463,22 @@ int gx_device_unsubclass(gx_device *dev)
 
     ref_count = dev->rc.ref_count;
     child = dev->child;
-    psubclass_data = dev->subclass_data;
+    psubclass_data = (generic_subclass_data *)dev->subclass_data;
     parent = dev->parent;
     dynamic = dev->stype_is_dynamic;
 
+    /* We need to account for the fact that we are removing ourselves from
+     * the device chain after a clist device has been pushed, due to a
+     * compositor action. Since we patched the clist 'create_compositor'
+     * method (and target device) when it was pushed.
+     * A point to note; we *don't* want to change the forwarding device's
+     * 'target', because when we copy the child up to replace 'this' device
+     * we do still want the forwarding device to point here. NB its the *child*
+     * device that goes away.
+     */
+    if (psubclass_data != NULL && psubclass_data->forwarding_dev != NULL && psubclass_data->saved_compositor_method)
+        psubclass_data->forwarding_dev->procs.create_compositor = psubclass_data->saved_compositor_method;
+
     /* If ths device's stype is dynamically allocated, keep a copy of it
      * in case we might need it.
      */
@@ -1460,15 +1494,28 @@ int gx_device_unsubclass(gx_device *dev)
 
     /* Copy the child device into ths device's memory */
     if (child) {
+        b_std = (gs_memory_struct_type_t *)dev->stype;
+        rc_decrement(dev->icc_struct, "unsubclass device");
+        rc_increment(child->icc_struct);
         memcpy(dev, child, child->stype->ssize);
         /* Patch back the 'stype' in the memory manager */
-        gs_set_object_type(child->memory, dev, child->stype);
+        gs_set_object_type(child->memory, dev, b_std);
+
+        dev->stype = b_std;
         /* The reference count of the subclassing device may have been changed
          * (eg graphics states pointing to it) after we subclassed the device. We
          * need to ensure that we do not overwrite this when we copy back the subclassed
          * device.
          */
         dev->rc.ref_count = ref_count;
+
+        /* If we have a chain of devices, make sure the chain beond the device we're unsubclassing
+         * doesn't get broken, we needd to detach the lower chain and reattach it at the new
+         * highest level
+         */
+        if (child->child)
+            child->child->parent = dev;
+        child->parent->child = child->child;
     }
 
     /* How can we have a subclass device with no child ? Simples; when we hit the end of job
@@ -1489,10 +1536,28 @@ int gx_device_unsubclass(gx_device *dev)
          * If this ever happens garbage collecton will eventually clean up the memory.
          */
         if (child->stype_is_dynamic) {
-            gs_memory_struct_type_t *b_std = (gs_memory_struct_type_t *)child->stype;
-            /* We definitley do *not* want to finalise the device, we just copied it up a level */
-            b_std->finalize = 0;
-            gs_free_object(dev->memory, child, "gx_unsubclass_device(device)");
+            /* Make sure that nothing will tyr to follow the device chain, just security here */
+            child->parent = NULL;
+            child->child = NULL;
+            /* Make certainthe memory will be freed, zap the reference count */
+            child->rc.ref_count = 0;
+            /* We *don't* want to run the finalize routine. This would free the stype and
+             * properly handle the icc_struct and PageList, but for devices with a custom
+             * finalize (eg psdcmyk) it might also free memory it had allocated, and we're
+             * still pointing at that memory in the parent.
+             * The indirection through a variable is just to get rid of const warnings.
+             */
+            b_std = (gs_memory_struct_type_t *)child->stype;
+            b_std->finalize = NULL;
+            /* Having patched the stype, we need to make sure the memory manager uses it.
+             * It keeps a copy in its own data structure, and would use that copy, which would
+             * mean it would call the finalize routine that we just patched out.
+             */
+            gs_set_object_type(dev->memory->stable_memory, child, b_std);
+            /* Now (finally) free the child memory */
+            gs_free_object(dev->memory->stable_memory, child, "gx_unsubclass_device(device)");
+            /* And the stype for it */
+            gs_free_const_object(dev->memory->non_gc_memory, b_std, "gs_device_unsubclass(stype)");
             child = 0;
         }
     }
