@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -151,6 +151,20 @@ gs_pattern1_make_pattern(gs_client_color * pcc,
     if (mem == 0)
         mem = gs_gstate_memory(pgs);
     pinst = (gs_pattern1_instance_t *)pcc->pattern;
+#ifdef PACIFY_VALGRIND
+    /* The following memset is required to avoid a valgrind warning
+     * in:
+     *   gs -I./gs/lib -sOutputFile=out.pgm -dMaxBitmap=10000
+     *      -sDEVICE=pgmraw -r300 -Z: -sDEFAULTPAPERSIZE=letter
+     *      -dNOPAUSE -dBATCH -K2000000 -dClusterJob -dJOBSERVER
+     *      tests_private/ps/ps3cet/11-14.PS
+     * Setting the individual elements of the structure directly is
+     * not enough, which leads me to believe that we are writing the
+     * entire struct out, padding and all.
+     */
+    memset(((char *)&inst) + sizeof(gs_pattern_instance_t), 0,
+           sizeof(inst) - sizeof(gs_pattern_instance_t));
+#endif
     *(gs_pattern_instance_t *)&inst = *(gs_pattern_instance_t *)pinst;
     saved = inst.saved;
     switch (pcp->PaintType) {
@@ -788,7 +802,7 @@ mask_PaintProc(const gs_client_color * pcolor, gs_gstate * pgs)
     gs_image_t_init_mask(&mask, true);
     mask.Width = pbitmap->size.x;
     mask.Height = pbitmap->size.y;
-    gs_image_init(pen, &mask, false, pgs);
+    gs_image_init(pen, &mask, false, false, pgs);
     return bitmap_paint(pen, (gs_data_image_t *) & mask, pbitmap, pgs);
 }
 static int
@@ -830,7 +844,7 @@ image_PaintProc(const gs_client_color * pcolor, gs_gstate * pgs)
         pcspace = ppmap->pcspace;
     code = gs_gsave(pgs);
     if (code < 0)
-        return code;
+        goto fail;
     code = gs_setcolorspace(pgs, pcspace);
     if (code < 0)
         return code;
@@ -856,13 +870,18 @@ image_PaintProc(const gs_client_color * pcolor, gs_gstate * pgs)
     if ( (code = gs_image_begin_typed( (const gs_image_common_t *)&image,
                                        pgs,
                                        false,
+                                       false,
                                        &pie )) >= 0 &&
          (code = gs_image_enum_init( pen,
                                      pie,
                                      (gs_data_image_t *)&image,
-                                     pgs )) >= 0      )
-        code = bitmap_paint(pen, (gs_data_image_t *) & image, pbitmap, pgs);
-    gs_grestore(pgs);
+                                     pgs )) >= 0 &&
+        (code = bitmap_paint(pen, (gs_data_image_t *) & image, pbitmap, pgs)) >= 0) {
+        return gs_grestore(pgs);
+    }
+
+fail:
+    gs_free_object(gs_gstate_memory(pgs), pen, "image_PaintProc");
     return code;
 }
 /* Finish painting any kind of bitmap pattern. */
@@ -1021,8 +1040,7 @@ static int pixmap_remap_image_pattern(const gs_client_color *pcc, gs_gstate *pgs
          */
         return_error(gs_error_Remap_Color);
     } else {
-        image_PaintProc(pcc, pgs);
-        return 0;
+        return image_PaintProc(pcc, pgs);
     }
 }
 
@@ -1124,7 +1142,7 @@ gs_makepixmappattern(
          * color, clear these so that there isn't an extra retained
          * reference to the Pattern object.
          */
-        gs_setgray(pinst->saved, 0.0);
+        code = gs_setgray(pinst->saved, 0.0);
 
     }
     gs_setmatrix(pgs, &smat);
@@ -1586,6 +1604,7 @@ typedef struct tile_trans_clist_info_s {
     int rowstride;
     int planestride;
     int n_chan; /* number of pixel planes including alpha */
+    bool has_tags;	/* extra plane for tags */
     int width;
     int height;
 } tile_trans_clist_info_t;
@@ -1716,6 +1735,8 @@ gx_dc_pattern_trans_write_raster(gx_color_tile *ptile, int64_t offset, byte *dat
     /* Everything that we need to handle the transparent tile */
 
     size = size_h + ptile->ttrans->n_chan * ptile->ttrans->planestride;
+    if (ptile->ttrans->has_tags)
+        size += ptile->ttrans->planestride;
 
     /* data is sent with NULL if the clist writer just wanted the size */
     if (data == NULL) {
@@ -1751,6 +1772,7 @@ gx_dc_pattern_trans_write_raster(gx_color_tile *ptile, int64_t offset, byte *dat
         /* Do the transparency information now */
         trans_info.height = ptile->ttrans->height;
         trans_info.n_chan = ptile->ttrans->n_chan;
+        trans_info.has_tags = ptile->ttrans->has_tags;
         trans_info.planestride = ptile->ttrans->planestride;
         trans_info.rect.p.x = ptile->ttrans->rect.p.x;
         trans_info.rect.p.y = ptile->ttrans->rect.p.y;
@@ -1768,11 +1790,11 @@ gx_dc_pattern_trans_write_raster(gx_color_tile *ptile, int64_t offset, byte *dat
         offset1 += sizeof(trans_info);
     }
 
-    /* Now do the transparency tile data itself.  Note that it may
-       be split up in the writing stage if it is large */
+    /* Now do the transparency tile data itself.  Note that it may be split up
+     * in the writing stage if it is large. The size include n_chan + the tag
+     * plane if this buffer has_tags. */
 
     /* check if we have written it all */
-
     if (offset1 <= size) {
         /* Get the most that we can write */
         int u = min(size, left);
@@ -1780,9 +1802,6 @@ gx_dc_pattern_trans_write_raster(gx_color_tile *ptile, int64_t offset, byte *dat
         /* copy that amount */
         ptr = ptile->ttrans->transbytes;
         memcpy(dp, ptr + (offset1 - size_h), u);
-        left -= u;
-        dp += u;
-        offset1 += u;
     }
     return 0;
 }
@@ -1816,10 +1835,10 @@ gx_dc_pattern_write(
         /* A special case for writing a known pattern :
            Just write the tile id. */
         gs_id id = ptile->id; /* Ensure sizeof(gs_id). */
-		if_debug2m('?', dev->memory,
-			"[v*] Writing trans tile ID into clist, uid = %ld id = %ld \n",
-			ptile->uid.id, ptile->id);
-		memcpy(dp, &ptile->id, sizeof(id));
+        if_debug2m('v', dev->memory,
+                   "[v*] Writing trans tile ID into clist, uid = %ld id = %ld \n",
+                   ptile->uid.id, ptile->id);
+        memcpy(dp, &ptile->id, sizeof(id));
         *psize = sizeof(gs_id);
         return 0;
     }
@@ -1827,12 +1846,12 @@ gx_dc_pattern_write(
     /* Check if pattern has transparency object
        If so then that is what we will stuff in
        the clist */
-	if (ptile->ttrans != NULL) {
-		if_debug2m('?', dev->memory,
-			"[v*] Writing trans tile into clist, uid = %ld id = %ld \n",
-			ptile->uid.id, ptile->id);
-		return gx_dc_pattern_trans_write_raster(ptile, offset, data, psize);
-	}
+        if (ptile->ttrans != NULL) {
+            if_debug2m('v', dev->memory,
+                       "[v*] Writing trans tile into clist, uid = %ld id = %ld \n",
+                       ptile->uid.id, ptile->id);
+            return gx_dc_pattern_trans_write_raster(ptile, offset, data, psize);
+        }
 
     if (ptile->cdev == NULL)
         return gx_dc_pattern_write_raster(ptile, offset, data, psize, dev);
@@ -1960,8 +1979,6 @@ gx_dc_pattern_read_raster(gx_color_tile *ptile, const gx_dc_serialized_tile_t *b
         memcpy(ptile->tmask.data +
                 (offset1 - sizeof(gx_dc_serialized_tile_t) - size_b - sizeof(gx_strip_bitmap)), dp, l);
         left -= l;
-        offset1 += l;
-        dp += l;
     }
     return size - left;
 }
@@ -1978,6 +1995,9 @@ gx_dc_pattern_read_trans_buff(gx_color_tile *ptile, int64_t offset,
     int data_size;
 
     data_size = trans_pat->planestride * trans_pat->n_chan;
+    if (trans_pat->has_tags)
+        data_size += trans_pat->planestride;
+
     /* Allocate the bytes */
     if (trans_pat->transbytes == NULL){
         trans_pat->transbytes = gs_alloc_bytes(mem, data_size, "gx_dc_pattern_read_raster");
@@ -1995,8 +2015,6 @@ gx_dc_pattern_read_trans_buff(gx_color_tile *ptile, int64_t offset,
                                     sizeof(tile_trans_clist_info_t), dp, u);
         trans_pat->transbytes = save;
         left -= u;
-        offset1 += u;
-        dp += u;
     }
      return size - left;
 }
@@ -2101,6 +2119,7 @@ gx_dc_pattern_read(
 
                 ptile->ttrans->height = trans_info.height;
                 ptile->ttrans->n_chan = trans_info.n_chan;
+                ptile->ttrans->has_tags = trans_info.has_tags;
                 ptile->ttrans->pdev14 = NULL;
                 ptile->ttrans->planestride = trans_info.planestride;
                 ptile->ttrans->rect.p.x = trans_info.rect.p.x;
@@ -2110,9 +2129,9 @@ gx_dc_pattern_read(
                 ptile->ttrans->rowstride = trans_info.rowstride;
                 ptile->ttrans->width = trans_info.width;
                 pdevc->type = &gx_dc_pattern_trans;
-				if_debug2m('?', pgs->memory,
-					"[v*] Reading trans tile from clist into cache, uid = %ld id = %ld \n",
-					ptile->uid.id, ptile->id);
+                if_debug2m('v', pgs->memory,
+                           "[v*] Reading trans tile from clist into cache, uid = %ld id = %ld \n",
+                           ptile->uid.id, ptile->id);
 
                 code = gx_dc_pattern_read_trans_buff(ptile, offset1, dp, left, mem);
                 if (code < 0)
@@ -2165,7 +2184,6 @@ gx_dc_pattern_read(
             return gx_dc_pattern_read_raster(ptile, NULL, offset1, dp, left, mem);
 
         size_b = ptile->tbits.size.x;
-        size_c = ptile->tbits.size.y;
     }
     if (offset1 <= sizeof(buf) + size_b) {
         l = min(left, size_b - (offset1 - sizeof(buf)));
@@ -2185,7 +2203,6 @@ gx_dc_pattern_read(
             return code;
         l = code;
         left -= l;
-        offset1 += l;
     }
     return size - left;
 }
