@@ -366,8 +366,8 @@ clist_fill_rectangle(gx_device * dev, int rx, int ry, int rwidth, int rheight,
             code = cmd_write_rect_cmd(cdev, re.pcls, cmd_op_fill_rect, rx, re.y,
                                       rwidth, re.height);
         }
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         re.y += re.height;
     } while (re.y < re.yend);
     return 0;
@@ -418,8 +418,8 @@ clist_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
             code = cmd_write_rect_hl_cmd(cdev, re.pcls, cmd_op_fill_rect_hl,
                                          rx, re.y, rwidth, re.height, false);
         }
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         re.y += re.height;
     } while (re.y < re.yend);
     return 0;
@@ -529,8 +529,8 @@ clist_write_fill_trapezoid(gx_device * dev,
             code = cmd_write_trapezoid_cmd(cdev, re.pcls, cmd_opv_fill_trapezoid, left, right,
                                       ybot, ytop, options, fa, c0, c1, c2, c3);
         }
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         re.y += re.height;
     } while (re.y < re.yend);
     return 0;
@@ -681,17 +681,15 @@ clist_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tile,
         RECT_STEP_INIT(re);
         re.pcls->color_usage.or |= color_usage;
         code = cmd_disable_lop(cdev, re.pcls);
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         /* Change the tile if needed */
         if (!cls_has_tile_id(cdev, re.pcls, tile->id, offset_temp)) {
             if (tile->id != gx_no_bitmap_id) {
                 code = clist_change_tile(cdev, re.pcls, tile, depth);
-                if (code == gs_error_VMerror && SET_BAND_CODE(code))	/* only VMerror exits here */
-                    return re.band_code;
-            } else
-                code = -1;
-            if (code < 0) {
+                if (code < 0)
+                    return code;
+            } else {
                 return_error(gs_error_unregistered);
             }
         }
@@ -712,8 +710,8 @@ clist_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tile,
                                          cmd_opv_ext_tile_rect_hl, rx, re.y,
                                          rwidth, re.height, true);
         }
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         re.y += re.height;
     } while (re.y < re.yend);
     return 0;
@@ -761,13 +759,11 @@ clist_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tile,
         RECT_STEP_INIT(re);
         re.pcls->color_usage.or |= color_usage;
         code = cmd_disable_lop(cdev, re.pcls);
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         if (!cls_has_tile_id(cdev, re.pcls, tile->id, offset_temp)) {
             if (tile->id != gx_no_bitmap_id) {
                 code = clist_change_tile(cdev, re.pcls, tile, depth);
-                if (code == gs_error_VMerror && SET_BAND_CODE(code))	/* only VMerror exits here */
-                    return re.band_code;
             } else
                 code = -1; /* Force the default implementation. Should never happen. */
             if (code < 0) {
@@ -777,8 +773,8 @@ clist_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tile,
                                                        rx, re.y, rwidth, re.height,
                                                        color0, color1,
                                                        px, py);
-                if (code < 0 && SET_BAND_CODE(code))
-                    return re.band_code;
+                if (code < 0)
+                    return code;
                 goto endr;
             }
         }
@@ -792,8 +788,8 @@ clist_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tile,
         if (code >= 0)
             code = cmd_write_rect_cmd(cdev, re.pcls, cmd_op_tile_rect, rx, re.y,
                                       rwidth, re.height);
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
 endr:;
         re.y += re.height;
     } while (re.y < re.yend);
@@ -846,8 +842,8 @@ clist_copy_mono(gx_device * dev,
             code = cmd_set_color0(cdev, re.pcls, color0);
         if (color1 != re.pcls->colors[1] && code >= 0)
             code = cmd_set_color1(cdev, re.pcls, color1);
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         /* Don't bother to check for a possible cache hit: */
         /* tile_rectangle and fill_mask handle those cases. */
 copy:{
@@ -867,8 +863,8 @@ copy:{
                                     1 << cmd_compress_rle :
                                     cmd_mask_compress_any),
                             &dp, &csize);
-        if (code < 0 && !(code == gs_error_limitcheck) && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0 && !(code == gs_error_limitcheck))
+            return code;
         compress = (uint)code;
         if (code < 0) {
             /* The bitmap was too large; split up the transfer. */
@@ -891,8 +887,8 @@ copy:{
                                            raster, gx_no_bitmap_id,
                                            rx + w2, re.y,
                                            w1 - w2, 1, color0, color1);
-                if (code < 0 && SET_BAND_CODE(code))
-                    return re.band_code;
+                if (code < 0)
+                    return code;
                 continue;
             }
         }
@@ -971,8 +967,8 @@ clist_copy_planes(gx_device * dev,
         code = cmd_disable_lop(cdev, re.pcls);
         if (code >= 0)
             code = cmd_disable_clip(cdev, re.pcls);
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         /* Don't bother to check for a possible cache hit: */
         /* tile_rectangle and fill_mask handle those cases. */
 
@@ -994,8 +990,8 @@ clist_copy_planes(gx_device * dev,
                                          raster, gx_no_bitmap_id,
                                          rx + w2, re.y,
                                          w1 - w2, 1, plane_height);
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
             continue;
         }
 
@@ -1044,8 +1040,8 @@ clist_copy_planes(gx_device * dev,
 
             csize += csize2;
         }
-        if (code < 0 && !(code == gs_error_limitcheck) && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0 && !(code == gs_error_limitcheck))
+            return code;
 
         re.pcls->rect = rect;
     } while ((re.y += re.height) < re.yend);
@@ -1096,15 +1092,15 @@ clist_copy_color(gx_device * dev,
         code = cmd_disable_lop(cdev, re.pcls);
         if (code >= 0)
             code = cmd_disable_clip(cdev, re.pcls);
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         if (re.pcls->color_is_alpha) {
             byte *dp;
 
             code =
                 set_cmd_put_op(&dp, cdev, re.pcls, cmd_opv_set_copy_color, 1);
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
             re.pcls->color_is_alpha = 0;
         }
 copy:{
@@ -1121,8 +1117,8 @@ copy:{
             code = cmd_put_bits(cdev, re.pcls, row, w1 * depth,
                                 re.height, raster, rsize,
                                 1 << cmd_compress_rle, &dp, &csize);
-            if (code < 0 && !(code == gs_error_limitcheck) && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0 && !(code == gs_error_limitcheck))
+                return code;
             compress = (uint)code;
             if (code < 0) {
                 /* The bitmap was too large; split up the transfer. */
@@ -1143,8 +1139,8 @@ copy:{
                         code = clist_copy_color(dev, row, dx + w2,
                                                 raster, gx_no_bitmap_id,
                                                 rx + w2, re.y, w1 - w2, 1);
-                    if (code < 0 && SET_BAND_CODE(code))
-                        return re.band_code;
+                    if (code < 0)
+                        return code;
                     continue;
                 }
             }
@@ -1213,14 +1209,14 @@ clist_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
         code = cmd_disable_lop(cdev, re.pcls);
         if (code >= 0)
             code = cmd_disable_clip(cdev, re.pcls);
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         if (!re.pcls->color_is_alpha) {
             byte *dp;
 
             code = set_cmd_put_op(&dp, cdev, re.pcls, cmd_opv_set_copy_alpha, 1);
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
             re.pcls->color_is_alpha = 1;
         }
         /* Set extended command for overload of copy_color_alpha with devn type */
@@ -1230,8 +1226,8 @@ clist_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
             code = set_cmd_put_op(&dp, cdev, re.pcls, cmd_opv_extend, 2);
             dp[1] = cmd_opv_ext_set_color_is_devn;
             dp += 2;
-            if (code < 0 && SET_BAND_CODE(code))
-                  return re.band_code;
+            if (code < 0)
+                  return code;
             re.pcls->color_is_alpha = 1;
         }
         /* Set the color */
@@ -1250,8 +1246,8 @@ copy:{
             code = cmd_put_bits(cdev, re.pcls, row, w1 << log2_depth,
                                 re.height, raster, rsize,
                                 1 << cmd_compress_rle, &dp, &csize);
-            if (code < 0 && !(code == gs_error_limitcheck) && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0 && !(code == gs_error_limitcheck))
+                return code;
             compress = (uint)code;
             if (code < 0) {
                 /* The bitmap was too large; split up the transfer. */
@@ -1273,8 +1269,8 @@ copy:{
                                                     raster, gx_no_bitmap_id,
                                                     rx + w2, re.y, w1 - w2, 1,
                                                     pdcolor, depth);
-                    if (code < 0 && SET_BAND_CODE(code))
-                        return re.band_code;
+                    if (code < 0)
+                        return code;
                     continue;
                 }
             }
@@ -1342,14 +1338,14 @@ clist_copy_alpha(gx_device * dev, const byte * data, int data_x,
         code = cmd_disable_lop(cdev, re.pcls);
         if (code >= 0)
             code = cmd_disable_clip(cdev, re.pcls);
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         if (!re.pcls->color_is_alpha) {
             byte *dp;
 
             code = set_cmd_put_op(&dp, cdev, re.pcls, cmd_opv_set_copy_alpha, 1);
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
             re.pcls->color_is_alpha = 1;
         }
         if (re.pcls->color_is_devn) {
@@ -1359,14 +1355,14 @@ clist_copy_alpha(gx_device * dev, const byte * data, int data_x,
             if (code >= 0)
                 code = set_cmd_put_op(&dp, cdev, re.pcls,
                                       cmd_opv_ext_unset_color_is_devn, 1);
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
             re.pcls->color_is_alpha = 1;
         }
         if (color != re.pcls->colors[1]) {
             code = cmd_set_color1(cdev, re.pcls, color);
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
         }
 copy:{
             gx_cmd_rect rect;
@@ -1382,8 +1378,8 @@ copy:{
             code = cmd_put_bits(cdev, re.pcls, row, w1 << log2_depth,
                                 re.height, raster, rsize,
                                 1 << cmd_compress_rle, &dp, &csize);
-            if (code < 0 && !(code == gs_error_limitcheck) && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0 && !(code == gs_error_limitcheck))
+                return code;
             compress = (uint)code;
             if (code < 0) {
                 /* The bitmap was too large; split up the transfer. */
@@ -1405,8 +1401,8 @@ copy:{
                                                 raster, gx_no_bitmap_id,
                                                 rx + w2, re.y, w1 - w2, 1,
                                                 color, depth);
-                    if (code < 0 && SET_BAND_CODE(code))
-                        return re.band_code;
+                    if (code < 0)
+                        return code;
                     continue;
                 }
             }
@@ -1564,8 +1560,8 @@ clist_strip_copy_rop2(gx_device * dev,
                     code = clist_change_tile(cdev, re.pcls, tiles,
                                             (tcolors != 0 ? 1 :
                                                  cdev->clist_color_info.depth));
-                    if (code < 0 && !(code == gs_error_limitcheck) && SET_BAND_CODE(code))
-                        return re.band_code;
+                    if (code < 0 && !(code == gs_error_limitcheck))
+                        return code;
                     if (code < 0) {
                         /*
                          * The error is a limitcheck: we have a tile that
@@ -1647,8 +1643,8 @@ clist_strip_copy_rop2(gx_device * dev,
                                     &line_tile, tcolors,
                                     rx, re.y + iy, rwidth, 1,
                                     new_phase, 0, lop, planar_height);
-                            if (code < 0 && SET_BAND_CODE(code))
-                                return re.band_code;
+                            if (code < 0)
+                                return code;
                         }
                         continue;
                     }
@@ -1656,8 +1652,8 @@ clist_strip_copy_rop2(gx_device * dev,
                         ((phase_y != re.pcls->tile_phase.y) && (tiles->rep_height > 1))) {
                         code = cmd_set_tile_phase(cdev, re.pcls, phase_x,
                                                   phase_y);
-                        if (code < 0 && SET_BAND_CODE(code))
-                            return re.band_code;
+                        if (code < 0)
+                            return code;
                     }
                 }
             }
@@ -1667,16 +1663,16 @@ clist_strip_copy_rop2(gx_device * dev,
                  cmd_set_tile_colors(cdev, re.pcls, tcolors[0], tcolors[1]) :
                  cmd_set_tile_colors(cdev, re.pcls, gx_no_color_index,
                                      gx_no_color_index));
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
         }
         code = 0;
         if (lop != re.pcls->lop)
             code = cmd_set_lop(cdev, re.pcls, lop);
         if (code >= 0)
             code = cmd_enable_lop(cdev, re.pcls);
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         /* Set lop_enabled to -1 so that fill_rectangle / copy_* */
         /* won't attempt to set it to 0. */
         re.pcls->lop_enabled = -1;
@@ -1697,8 +1693,8 @@ clist_strip_copy_rop2(gx_device * dev,
                                     rx, re.y, rwidth, re.height);
         }
         re.pcls->lop_enabled = 1;
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
     } while ((re.y += re.height) < re.yend);
     return 0;
 }
