@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2014 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -344,6 +344,7 @@ const gs_malloc_memory_t minimal_memory = {
     0  /* max used */
 };
 
+int cmpstringp(const void *p1, const void *p2);
 void put_uint32(FILE *out, const unsigned int q);
 void put_bytes_padded(FILE *out, unsigned char *p, unsigned int len);
 void inode_clear(romfs_inode* node);
@@ -1542,6 +1543,15 @@ static unsigned long pscompact_getcompactedblock(pscompstate *psc, unsigned char
     return out-ubuf;
 }
 
+int cmpstringp(const void *p1, const void *p2)
+{
+   /* The actual arguments to this function are "pointers to
+      pointers to char", but strcmp(3) arguments are "pointers
+      to char", hence the following cast plus dereference */
+
+   return strcmp(* (char * const *) p1, * (char * const *) p2);
+}
+
 /* This relies on the gp_enumerate_* which should not return directories, nor	*/
 /* should it recurse into directories (unlike Adobe's implementation)		*/
 /* paths are checked to see if they are an ordinary file or a path		*/
@@ -1549,7 +1559,7 @@ void process_path(char *path, const char *os_prefix, const char *rom_prefix,
                   Xlist_element *Xlist_head, int compression,
                   int compaction, int *inode_count, int *totlen, FILE *out)
 {
-    int namelen, excluded, save_count=*inode_count;
+    int i, namelen, excluded, save_count=*inode_count;
     Xlist_element *Xlist_scan;
     char *prefixed_path;
     char *found_path, *rom_filename;
@@ -1561,6 +1571,8 @@ void process_path(char *path, const char *os_prefix, const char *rom_prefix,
     FILE *in;
     unsigned long psc_len;
     pscompstate psc = { 0 };
+    unsigned long numfiles = 0;
+    char **foundfiles = NULL, *temp;
 
     prefixed_path = malloc(PATH_STR_LEN);
     found_path = malloc(PATH_STR_LEN);
@@ -1598,19 +1610,40 @@ void process_path(char *path, const char *os_prefix, const char *rom_prefix,
         if (excluded)
             continue;
 
+        numfiles++;
+        temp = realloc(foundfiles, sizeof(char *) * numfiles);
+        if (temp == NULL) {
+            free(cbuf);
+            free(ubuf);
+            free(found_path);
+            free(foundfiles);
+            free(prefixed_path);
+            free(rom_filename);
+            printf("realloc failed in process_path.\n");
+            exit(1);
+        }
+        foundfiles = (char **)temp;
+        foundfiles[numfiles - 1] = strdup(found_path);
+    }
+
+    qsort(foundfiles, numfiles, sizeof(char *), cmpstringp);
+
+    for (i = 0; i < numfiles; i++) {
+        char *fpath = foundfiles[i];
+
         /* process a file */
         node = calloc(1, sizeof(romfs_inode));
         /* get info for this file */
-        in = fopen(found_path, "rb");
+        in = fopen(fpath, "rb");
         if (in == NULL) {
-            printf("unable to open file for processing: %s\n", found_path);
+            printf("unable to open file for processing: %s\n", fpath);
             continue;
         }
-        /* printf("compacting %s\n", found_path); */
+        /* printf("compacting %s\n", fpath); */
         /* rom_filename + strlen(rom_prefix) is first char after the new prefix we want to add */
-        /* found_path + strlen(os_prefix) is the file name after the -P prefix */
+        /* fpath + strlen(os_prefix) is the file name after the -P prefix */
         rom_filename[strlen(rom_prefix)] = 0;		/* truncate afater prefix */
-        strcat(rom_filename, found_path + strlen(os_prefix));
+        strcat(rom_filename, fpath + strlen(os_prefix));
         node->name = rom_filename;	/* without -P prefix, with -d rom_prefix */
         fseek(in, 0, SEEK_END);
         node->disc_length = node->length = ftell(in);
@@ -1618,8 +1651,8 @@ void process_path(char *path, const char *os_prefix, const char *rom_prefix,
         node->data_lengths = calloc(blocks, sizeof(*node->data_lengths));
         node->data = calloc(blocks, sizeof(*node->data));
         fclose(in);
-        in = fopen(found_path, "rb");
-        ulen = strlen(found_path);
+        in = fopen(fpath, "rb");
+        ulen = strlen(fpath);
         block = 0;
         psc_len = 0;
         if (compaction)
@@ -1651,7 +1684,7 @@ void process_path(char *path, const char *os_prefix, const char *rom_prefix,
         fclose(in);
         if (compaction) {
             /* printf("%s: Compaction saved %d bytes (before compression)\n",
-             *        found_path, node->length - psc_len); */
+             *        fpath, node->length - psc_len); */
             pscompact_end(&psc);
             node->length = psc_len;
         }
@@ -1661,10 +1694,12 @@ void process_path(char *path, const char *os_prefix, const char *rom_prefix,
         inode_clear(node);
         free(node);
         (*inode_count)++;
+        free(fpath);
     }
     free(cbuf);
     free(ubuf);
     free(found_path);
+    free(foundfiles);
     free(prefixed_path);
     free(rom_filename);
     if (save_count == *inode_count) {
@@ -2280,6 +2315,8 @@ main(int argc, char *argv[])
     int compaction = 0;
     Xlist_element *Xlist_scan = NULL, *Xlist_head = NULL;
     char pa[PATH_STR_LEN];
+    time_t buildtime = 0;
+    char* env_source_date_epoch;
 
     memset(pa, 0x00, PATH_STR_LEN);
 
@@ -2336,7 +2373,13 @@ main(int argc, char *argv[])
 #endif
     fprintf(out,"\n#include \"stdint_.h\"\n");
     fprintf(out,"\n#include \"time_.h\"\n\n");
-    fprintf(out,"    time_t gs_romfs_buildtime = %ld;\n\n", time(NULL));
+
+    if ((env_source_date_epoch = getenv("SOURCE_DATE_EPOCH"))) {
+        buildtime = strtoul(env_source_date_epoch, NULL, 10);
+    }
+    if (!buildtime)
+        buildtime = time(NULL);
+    fprintf(out,"    time_t gs_romfs_buildtime = %ld;\n\n", buildtime);
 
     /* process the remaining arguments (options interspersed with paths) */
     for (; atarg < argc; atarg++) {
