@@ -48,35 +48,40 @@ jbig2_arith_iaid_ctx_new(Jbig2Ctx *ctx, int SBSYMCODELEN)
     int ctx_size = 1 << SBSYMCODELEN;
 
     if (result == NULL) {
-        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1, "failed to allocate storage in jbig2_arith_iaid_ctx_new");
-        return result;
+        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1, "failed to allocate IAID arithmetic coding state");
+        return NULL;
     }
 
     result->SBSYMCODELEN = SBSYMCODELEN;
     result->IAIDx = jbig2_new(ctx, Jbig2ArithCx, ctx_size);
-    if (result->IAIDx != NULL) {
-        memset(result->IAIDx, 0, ctx_size);
-    } else {
-        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1, "failed to allocate symbol ID storage in jbig2_arith_iaid_ctx_new");
+    if (result->IAIDx == NULL)
+    {
+        jbig2_free(ctx->allocator, result);
+        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1, "failed to allocate symbol ID in IAID arithmetic coding state");
+        return NULL;
     }
 
+    memset(result->IAIDx, 0, ctx_size);
     return result;
 }
 
 /* A.3 */
 /* Return value: -1 on error, 0 on normal value */
 int
-jbig2_arith_iaid_decode(Jbig2ArithIaidCtx *ctx, Jbig2ArithState *as, int32_t *p_result)
+jbig2_arith_iaid_decode(Jbig2Ctx *ctx, Jbig2ArithIaidCtx *actx, Jbig2ArithState *as, int32_t *p_result)
 {
-    Jbig2ArithCx *IAIDx = ctx->IAIDx;
-    int SBSYMCODELEN = ctx->SBSYMCODELEN;
+    Jbig2ArithCx *IAIDx = actx->IAIDx;
+    int SBSYMCODELEN = actx->SBSYMCODELEN;
     int PREV = 1;
     int D;
     int i;
+    int code = 0;
 
     /* A.3 (2) */
     for (i = 0; i < SBSYMCODELEN; i++) {
-        D = jbig2_arith_decode(as, &IAIDx[PREV]);
+        D = jbig2_arith_decode(as, &IAIDx[PREV], &code);
+        if (code)
+            return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1, "failed to decode IAIDx code");
 #ifdef VERBOSE
         fprintf(stderr, "IAID%x: D = %d\n", PREV, D);
 #endif
