@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /*
@@ -30,21 +30,25 @@
 #include "jbig2_priv.h"
 #include "jbig2_arith.h"
 #include "jbig2_generic.h"
-#include "jbig2_mmr.h"
 #include "jbig2_image.h"
 #include "jbig2_halftone.h"
+#include "jbig2_mmr.h"
+#include "jbig2_page.h"
+#include "jbig2_segment.h"
 
 /**
  * jbig2_hd_new: create a new dictionary from a collective bitmap
  */
-Jbig2PatternDict *
+static Jbig2PatternDict *
 jbig2_hd_new(Jbig2Ctx *ctx, const Jbig2PatternDictParams *params, Jbig2Image *image)
 {
     Jbig2PatternDict *new;
-    const int N = params->GRAYMAX + 1;
-    const int HPW = params->HDPW;
-    const int HPH = params->HDPH;
+    const uint32_t N = params->GRAYMAX + 1;
+    const uint32_t HPW = params->HDPW;
+    const uint32_t HPH = params->HDPH;
+    int code;
     int i;
+    int j;
 
     /* allocate a new struct */
     new = jbig2_new(ctx, Jbig2PatternDict, 1);
@@ -63,8 +67,6 @@ jbig2_hd_new(Jbig2Ctx *ctx, const Jbig2PatternDictParams *params, Jbig2Image *im
         for (i = 0; i < N; i++) {
             new->patterns[i] = jbig2_image_new(ctx, HPW, HPH);
             if (new->patterns[i] == NULL) {
-                int j;
-
                 jbig2_error(ctx, JBIG2_SEVERITY_WARNING, -1, "failed to allocate pattern element image");
                 for (j = 0; j < i; j++)
                     jbig2_free(ctx->allocator, new->patterns[j]);
@@ -72,9 +74,16 @@ jbig2_hd_new(Jbig2Ctx *ctx, const Jbig2PatternDictParams *params, Jbig2Image *im
                 return NULL;
             }
             /* compose with the REPLACE operator; the source
-               will be clipped to the destintion, selecting the
+               will be clipped to the destination, selecting the
                proper sub image */
-            jbig2_image_compose(ctx, new->patterns[i], image, -i * HPW, 0, JBIG2_COMPOSE_REPLACE);
+            code = jbig2_image_compose(ctx, new->patterns[i], image, -i * HPW, 0, JBIG2_COMPOSE_REPLACE);
+            if (code < 0) {
+                jbig2_error(ctx, JBIG2_SEVERITY_WARNING, -1, "failed to compose image into collective bitmap dictionary");
+                for (j = 0; j < i; j++)
+                    jbig2_free(ctx->allocator, new->patterns[j]);
+                jbig2_free(ctx->allocator, new);
+                return NULL;
+            }
         }
     } else {
         jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1, "failed to allocate collective bitmap dictionary");
@@ -93,8 +102,8 @@ jbig2_hd_release(Jbig2Ctx *ctx, Jbig2PatternDict *dict)
 
     if (dict == NULL)
         return;
-    for (i = 0; i < dict->n_patterns; i++)
-        if (dict->patterns[i])
+    if (dict->patterns != NULL)
+        for (i = 0; i < dict->n_patterns; i++)
             jbig2_image_release(ctx, dict->patterns[i]);
     jbig2_free(ctx->allocator, dict->patterns);
     jbig2_free(ctx->allocator, dict);
@@ -108,9 +117,9 @@ jbig2_hd_release(Jbig2Ctx *ctx, Jbig2PatternDict *dict)
  * @params: parameters from the pattern dictionary header
  * @data: pointer to text region data to be decoded
  * @size: length of text region data
- * @GB_stats: artimetic coding context to use
+ * @GB_stats: arithmetic coding context to use
  *
- * Implements the patten dictionary decoding proceedure
+ * Implements the pattern dictionary decoding procedure
  * described in section 6.7 of the JBIG2 spec.
  *
  * returns: a pointer to the resulting dictionary on success
@@ -128,7 +137,7 @@ jbig2_decode_pattern_dict(Jbig2Ctx *ctx, Jbig2Segment *segment,
     /* allocate the collective image */
     image = jbig2_image_new(ctx, params->HDPW * (params->GRAYMAX + 1), params->HDPH);
     if (image == NULL) {
-        jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to allocate collective bitmap for halftone dict!");
+        jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to allocate collective bitmap for halftone dictionary");
         return NULL;
     }
 
@@ -157,18 +166,20 @@ jbig2_decode_pattern_dict(Jbig2Ctx *ctx, Jbig2Segment *segment,
             if (as != NULL) {
                 code = jbig2_decode_generic_region(ctx, segment, &rparams, as, image, GB_stats);
             } else {
-                code = jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to allocate storage for as in halftone dict!");
+                code = jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to allocate arithmetic coding state when handling halftone dictionary");
             }
 
             jbig2_free(ctx->allocator, as);
             jbig2_word_stream_buf_free(ctx, ws);
         } else {
-            code = jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to allocate storage for ws in halftone dict!");
+            code = jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to allocate word stream when handling halftone dictionary");
         }
     }
 
     if (code == 0)
         hd = jbig2_hd_new(ctx, params, image);
+    else
+        jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to decode immediate generic region");
     jbig2_image_release(ctx, image);
 
     return hd;
@@ -185,7 +196,7 @@ jbig2_pattern_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment, const byte *segme
 
     /* 7.4.4.1 - Data header */
     if (segment->data_length < 7) {
-        return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "Segment too short");
+        return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "segment too short");
     }
     flags = segment_data[0];
     params.HDMMR = flags & 1;
@@ -202,7 +213,7 @@ jbig2_pattern_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment, const byte *segme
         jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "HDTEMPLATE is %d when HDMMR is %d, contrary to spec", params.HDTEMPLATE, params.HDMMR);
     }
     if (flags & 0xf8) {
-        jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "Reserved flag bits non-zero");
+        jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "reserved flag bits non-zero");
     }
 
     /* 7.4.4.2 */
@@ -211,10 +222,8 @@ jbig2_pattern_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment, const byte *segme
         int stats_size = jbig2_generic_stats_size(ctx, params.HDTEMPLATE);
 
         GB_stats = jbig2_new(ctx, Jbig2ArithCx, stats_size);
-        if (GB_stats == NULL) {
-            jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "failed to allocate GB_stats in pattern dictionary");
-            return 0;
-        }
+        if (GB_stats == NULL)
+            return jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to allocate arithmetic coding state when handling pattern dictionary");
         memset(GB_stats, 0, stats_size);
     }
 
@@ -241,7 +250,7 @@ jbig2_pattern_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment, const byte *segme
  * @GSBPP: number of bitplanes/Jbig2Images to use
  * @GSKIP: mask indicating which values should be skipped
  * @GSTEMPLATE: template used to code the gray-scale bitplanes
- * @GB_stats: artimetic coding context to use
+ * @GB_stats: arithmetic coding context to use
  *
  * Implements the decoding a gray-scale image described in
  * annex C.5. This is part of the halftone region decoding.
@@ -249,16 +258,16 @@ jbig2_pattern_dictionary(Jbig2Ctx *ctx, Jbig2Segment *segment, const byte *segme
  * returns: array of gray-scale values with GSW x GSH width/height
  *          0 on failure
  **/
-uint8_t **
+static uint16_t **
 jbig2_decode_gray_scale_image(Jbig2Ctx *ctx, Jbig2Segment *segment,
                               const byte *data, const size_t size,
                               bool GSMMR, uint32_t GSW, uint32_t GSH,
                               uint32_t GSBPP, bool GSUSESKIP, Jbig2Image *GSKIP, int GSTEMPLATE, Jbig2ArithCx *GB_stats)
 {
-    uint8_t **GSVALS = NULL;
+    uint16_t **GSVALS = NULL;
     size_t consumed_bytes = 0;
-    int i, j, code, stride;
-    int x, y;
+    uint32_t i, j, stride, x, y;
+    int code;
     Jbig2Image **GSPLANES;
     Jbig2GenericRegionParams rparams;
     Jbig2WordStream *ws = NULL;
@@ -274,11 +283,10 @@ jbig2_decode_gray_scale_image(Jbig2Ctx *ctx, Jbig2Segment *segment,
     for (i = 0; i < GSBPP; ++i) {
         GSPLANES[i] = jbig2_image_new(ctx, GSW, GSH);
         if (GSPLANES[i] == NULL) {
-            jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "failed to allocate %dx%d image for GSPLANES", GSW, GSH);
+            jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to allocate %dx%d image for GSPLANES", GSW, GSH);
             /* free already allocated */
-            for (j = i - 1; j >= 0; --j) {
-                jbig2_image_release(ctx, GSPLANES[j]);
-            }
+            for (j = i; j > 0;)
+                jbig2_image_release(ctx, GSPLANES[--j]);
             jbig2_free(ctx->allocator, GSPLANES);
             return NULL;
         }
@@ -290,6 +298,7 @@ jbig2_decode_gray_scale_image(Jbig2Ctx *ctx, Jbig2Segment *segment,
     rparams.GBTEMPLATE = GSTEMPLATE;
     rparams.TPGDON = 0;
     rparams.USESKIP = GSUSESKIP;
+    rparams.SKIP = GSKIP;
     rparams.gbat[0] = (GSTEMPLATE <= 1 ? 3 : 2);
     rparams.gbat[1] = -1;
     rparams.gbat[2] = -3;
@@ -304,28 +313,28 @@ jbig2_decode_gray_scale_image(Jbig2Ctx *ctx, Jbig2Segment *segment,
     } else {
         ws = jbig2_word_stream_buf_new(ctx, data, size);
         if (ws == NULL) {
-            jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "failed to allocate ws in jbig2_decode_gray_scale_image");
+            jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to allocate word stream when decoding gray scale image");
             goto cleanup;
         }
 
         as = jbig2_arith_new(ctx, ws);
         if (as == NULL) {
-            jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "failed to allocate as in jbig2_decode_gray_scale_image");
+            jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to allocate arithmetic coding state when decoding gray scale image");
             goto cleanup;
         }
 
         code = jbig2_decode_generic_region(ctx, segment, &rparams, as, GSPLANES[GSBPP - 1], GB_stats);
-
     }
     if (code != 0) {
-        jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "error decoding GSPLANES for halftone image");
+        jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "error decoding GSPLANES for halftone image");
         goto cleanup;
     }
 
     /* C.5 step 2. Set j = GSBPP-2 */
-    j = GSBPP - 2;
+    j = GSBPP - 1;
     /* C.5 step 3. decode loop */
-    while (j >= 0) {
+    while (j > 0) {
+        j--;
         /*  C.5 step 3. (a) */
         if (GSMMR) {
             code = jbig2_decode_halftone_mmr(ctx, &rparams, data + consumed_bytes, size - consumed_bytes, GSPLANES[j], &consumed_bytes);
@@ -333,35 +342,33 @@ jbig2_decode_gray_scale_image(Jbig2Ctx *ctx, Jbig2Segment *segment,
             code = jbig2_decode_generic_region(ctx, segment, &rparams, as, GSPLANES[j], GB_stats);
         }
         if (code != 0) {
-            jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "error decoding GSPLANES for halftone image");
+            jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to decode GSPLANES for halftone image");
             goto cleanup;
         }
 
         /* C.5 step 3. (b):
          * for each [x,y]
          * GSPLANES[j][x][y] = GSPLANES[j+1][x][y] XOR GSPLANES[j][x][y] */
-        stride = GSPLANES[0]->stride;
+        stride = GSPLANES[j]->stride;
         for (i = 0; i < stride * GSH; ++i)
             GSPLANES[j]->data[i] ^= GSPLANES[j + 1]->data[i];
 
         /*  C.5 step 3. (c) */
-        --j;
     }
 
     /* allocate GSVALS */
-    GSVALS = jbig2_new(ctx, uint8_t *, GSW);
+    GSVALS = jbig2_new(ctx, uint16_t *, GSW);
     if (GSVALS == NULL) {
         jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "failed to allocate GSVALS: %d bytes", GSW);
         goto cleanup;
     }
     for (i = 0; i < GSW; ++i) {
-        GSVALS[i] = jbig2_new(ctx, uint8_t, GSH);
+        GSVALS[i] = jbig2_new(ctx, uint16_t, GSH);
         if (GSVALS[i] == NULL) {
             jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "failed to allocate GSVALS: %d bytes", GSH * GSW);
             /* free already allocated */
-            for (j = i - 1; j >= 0; --j) {
-                jbig2_free(ctx->allocator, GSVALS[j]);
-            }
+            for (j = i; j > 0;)
+                jbig2_free(ctx->allocator, GSVALS[--j]);
             jbig2_free(ctx->allocator, GSVALS);
             GSVALS = NULL;
             goto cleanup;
@@ -403,7 +410,7 @@ cleanup:
  * returns: pattern dictionary
  *          0 if search failed
  **/
-Jbig2PatternDict *
+static Jbig2PatternDict *
 jbig2_decode_ht_region_get_hpats(Jbig2Ctx *ctx, Jbig2Segment *segment)
 {
     int index = 0;
@@ -433,84 +440,107 @@ jbig2_decode_ht_region_get_hpats(Jbig2Ctx *ctx, Jbig2Segment *segment)
  * @params: parameters
  * @data: pointer to halftone region data to be decoded
  * @size: length of halftone region data
- * @GB_stats: artimetic coding context to use
+ * @GB_stats: arithmetic coding context to use
  *
- * Implements the halftone region decoding proceedure
+ * Implements the halftone region decoding procedure
  * described in section 6.6.5 of the JBIG2 spec.
  *
  * returns: 0 on success
  *         <0 on failure
  **/
-int
+static int
 jbig2_decode_halftone_region(Jbig2Ctx *ctx, Jbig2Segment *segment,
                              Jbig2HalftoneRegionParams *params, const byte *data, const size_t size, Jbig2Image *image, Jbig2ArithCx *GB_stats)
 {
     uint32_t HBPP;
     uint32_t HNUMPATS;
-    uint8_t **GI;
+    uint16_t **GI = NULL;
     Jbig2Image *HSKIP = NULL;
     Jbig2PatternDict *HPATS;
-    int i;
-    uint32_t mg, ng;
+    uint32_t i;
+    int32_t mg, ng;
     int32_t x, y;
-    uint8_t gray_val;
+    uint16_t gray_val;
+    int code = 0;
+
+    /* We need the patterns used in this region, get them from the referred pattern dictionary */
+    HPATS = jbig2_decode_ht_region_get_hpats(ctx, segment);
+    if (!HPATS) {
+        code = jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "no pattern dictionary found, skipping halftone image");
+        goto cleanup;
+    }
 
     /* 6.6.5 point 1. Fill bitmap with HDEFPIXEL */
     memset(image->data, params->HDEFPIXEL, image->stride * image->height);
 
-    /* 6.6.5 point 2. compute HSKIP */
+    /* 6.6.5 point 2. compute HSKIP according to 6.6.5.1 */
     if (params->HENABLESKIP == 1) {
-        jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "unhandled option HENABLESKIP");
+        HSKIP = jbig2_image_new(ctx, params->HGW, params->HGH);
+        if (HSKIP == NULL)
+            return jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to allocate skip image");
+
+        for (mg = 0; mg < params->HGH; ++mg) {
+            for (ng = 0; ng < params->HGW; ++ng) {
+                x = (params->HGX + mg * (int32_t) params->HRY + ng * (int32_t) params->HRX) >> 8;
+                y = (params->HGY + mg * (int32_t) params->HRX - ng * (int32_t) params->HRY) >> 8;
+
+                if (x + HPATS->HPW <= 0 || x >= (int32_t) image->width || y + HPATS->HPH <= 0 || y >= (int32_t) image->height) {
+                    jbig2_image_set_pixel(HSKIP, ng, mg, 1);
+                } else {
+                    jbig2_image_set_pixel(HSKIP, ng, mg, 0);
+                }
+            }
+        }
     }
 
-    /* 6.6.5 point 3. set HBPP to ceil(log2(HNUMPATS)):
-     * we need the number of patterns used in this region (HNUMPATS)
-     * get it from referred pattern dictionary */
-
-    HPATS = jbig2_decode_ht_region_get_hpats(ctx, segment);
-    if (!HPATS) {
-        jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "no pattern dictionary found, skipping halftone image");
-        return -1;
-    }
+    /* 6.6.5 point 3. set HBPP to ceil(log2(HNUMPATS)): */
     HNUMPATS = HPATS->n_patterns;
-
-    /* calculate ceil(log2(HNUMPATS)) */
     HBPP = 0;
-    while (HNUMPATS > (1 << ++HBPP));
+    while (HNUMPATS > (1U << ++HBPP));
+    if (HBPP > 16) {
+        code = jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "HBPP is larger than supported (%u)", HBPP);
+        goto cleanup;
+    }
 
     /* 6.6.5 point 4. decode gray-scale image as mentioned in annex C */
     GI = jbig2_decode_gray_scale_image(ctx, segment, data, size,
                                        params->HMMR, params->HGW, params->HGH, HBPP, params->HENABLESKIP, HSKIP, params->HTEMPLATE, GB_stats);
-
     if (!GI) {
-        jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "unable to acquire gray-scale image, skipping halftone image");
-        return -1;
+        code = jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "unable to acquire gray-scale image, skipping halftone image");
+        goto cleanup;
     }
 
     /* 6.6.5 point 5. place patterns with procedure mentioned in 6.6.5.2 */
     for (mg = 0; mg < params->HGH; ++mg) {
         for (ng = 0; ng < params->HGW; ++ng) {
-            x = (params->HGX + mg * params->HRY + ng * params->HRX) >> 8;
-            y = (params->HGY + mg * params->HRX - ng * params->HRY) >> 8;
+            x = (params->HGX + mg * (int32_t) params->HRY + ng * (int32_t) params->HRX) >> 8;
+            y = (params->HGY + mg * (int32_t) params->HRX - ng * (int32_t) params->HRY) >> 8;
 
             /* prevent pattern index >= HNUMPATS */
             gray_val = GI[ng][mg];
             if (gray_val >= HNUMPATS) {
-                jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "gray-scale image uses value %d which larger than pattern dictionary", gray_val);
-                /* use highest aviable pattern */
+                jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "gray-scale index %d out of range, using largest index", gray_val);
+                /* use highest available pattern */
                 gray_val = HNUMPATS - 1;
             }
-            jbig2_image_compose(ctx, image, HPATS->patterns[gray_val], x, y, params->op);
+            code = jbig2_image_compose(ctx, image, HPATS->patterns[gray_val], x, y, params->HCOMBOP);
+            if (code < 0) {
+                code = jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to compose pattern with gray-scale image");
+                goto cleanup;
+            }
         }
     }
 
-    /* free GI */
-    for (i = 0; i < params->HGW; ++i) {
-        jbig2_free(ctx->allocator, GI[i]);
+cleanup:
+    if (GI) {
+        for (i = 0; i < params->HGW; ++i) {
+            jbig2_free(ctx->allocator, GI[i]);
+        }
     }
     jbig2_free(ctx->allocator, GI);
+    jbig2_image_release(ctx, HSKIP);
 
-    return 0;
+    return code;
 }
 
 /**
@@ -535,17 +565,17 @@ jbig2_halftone_region(Jbig2Ctx *ctx, Jbig2Segment *segment, const byte *segment_
     if (segment->data_length < 18)
         goto too_short;
 
-    /* 7.4.5.1.1 */
+    /* 7.4.5.1.1 Figure 42 */
     params.flags = segment_data[offset];
     params.HMMR = params.flags & 1;
     params.HTEMPLATE = (params.flags & 6) >> 1;
     params.HENABLESKIP = (params.flags & 8) >> 3;
-    params.op = (Jbig2ComposeOp)((params.flags & 0x70) >> 4);
+    params.HCOMBOP = (Jbig2ComposeOp)((params.flags & 0x70) >> 4);
     params.HDEFPIXEL = (params.flags & 0x80) >> 7;
     offset += 1;
 
     jbig2_error(ctx, JBIG2_SEVERITY_INFO, segment->number,
-                "halftone region: %d x %d @ (%x,%d) flags=%02x", region_info.width, region_info.height, region_info.x, region_info.y, params.flags);
+                "halftone region: %u x %u @ (%u, %u), flags = %02x", region_info.width, region_info.height, region_info.x, region_info.y, params.flags);
 
     if (params.HMMR && params.HTEMPLATE) {
         jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "HTEMPLATE is %d when HMMR is %d, contrary to spec", params.HTEMPLATE, params.HMMR);
@@ -554,7 +584,7 @@ jbig2_halftone_region(Jbig2Ctx *ctx, Jbig2Segment *segment, const byte *segment_
         jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "HENABLESKIP is %d when HMMR is %d, contrary to spec", params.HENABLESKIP, params.HMMR);
     }
 
-    /* Figure 43 */
+    /* 7.4.5.1.2 Figure 43 */
     if (segment->data_length - offset < 16)
         goto too_short;
     params.HGW = jbig2_get_uint32(segment_data + offset);
@@ -563,7 +593,7 @@ jbig2_halftone_region(Jbig2Ctx *ctx, Jbig2Segment *segment, const byte *segment_
     params.HGY = jbig2_get_int32(segment_data + offset + 12);
     offset += 16;
 
-    /* Figure 44 */
+    /* 7.4.5.1.3 Figure 44 */
     if (segment->data_length - offset < 4)
         goto too_short;
     params.HRX = jbig2_get_uint16(segment_data + offset);
@@ -571,18 +601,21 @@ jbig2_halftone_region(Jbig2Ctx *ctx, Jbig2Segment *segment, const byte *segment_
     offset += 4;
 
     jbig2_error(ctx, JBIG2_SEVERITY_INFO, segment->number,
-                " grid %d x %d @ (%d.%d,%d.%d) vector (%d.%d,%d.%d)",
+                "grid %d x %d @ (%d.%d,%d.%d) vector (%d.%d,%d.%d)",
                 params.HGW, params.HGH,
-                params.HGX >> 8, params.HGX & 0xff, params.HGY >> 8, params.HGY & 0xff, params.HRX >> 8, params.HRX & 0xff, params.HRY >> 8, params.HRY & 0xff);
+                params.HGX >> 8, params.HGX & 0xff,
+                params.HGY >> 8, params.HGY & 0xff,
+                params.HRX >> 8, params.HRX & 0xff,
+                params.HRY >> 8, params.HRY & 0xff);
 
-    /* 7.4.5.2.2 */
+    /* 7.4.5.2 */
     if (!params.HMMR) {
         /* allocate and zero arithmetic coding stats */
         int stats_size = jbig2_generic_stats_size(ctx, params.HTEMPLATE);
 
         GB_stats = jbig2_new(ctx, Jbig2ArithCx, stats_size);
         if (GB_stats == NULL) {
-            return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "failed to allocate GB_stats in halftone region");
+            return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "failed to allocate arithmetic decoder states in halftone region");
         }
         memset(GB_stats, 0, stats_size);
     }
@@ -590,21 +623,31 @@ jbig2_halftone_region(Jbig2Ctx *ctx, Jbig2Segment *segment, const byte *segment_
     image = jbig2_image_new(ctx, region_info.width, region_info.height);
     if (image == NULL) {
         jbig2_free(ctx->allocator, GB_stats);
-        return jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "unable to allocate halftone image");
+        return jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to allocate halftone image");
     }
 
     code = jbig2_decode_halftone_region(ctx, segment, &params, segment_data + offset, segment->data_length - offset, image, GB_stats);
+    if (code < 0) {
+        jbig2_image_release(ctx, image);
+        jbig2_free(ctx->allocator, GB_stats);
+        return jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "failed to decode halftone region");
+    }
 
     /* todo: retain GB_stats? */
     if (!params.HMMR) {
         jbig2_free(ctx->allocator, GB_stats);
     }
 
-    jbig2_page_add_result(ctx, &ctx->pages[ctx->current_page], image, region_info.x, region_info.y, region_info.op);
+    code = jbig2_page_add_result(ctx, &ctx->pages[ctx->current_page], image, region_info.x, region_info.y, region_info.op);
+    if (code < 0) {
+        jbig2_image_release(ctx, image);
+        return jbig2_error(ctx, JBIG2_SEVERITY_WARNING, segment->number, "unable to add halftone region to page");
+    }
+
     jbig2_image_release(ctx, image);
 
     return code;
 
 too_short:
-    return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "Segment too short");
+    return jbig2_error(ctx, JBIG2_SEVERITY_FATAL, segment->number, "segment too short");
 }
