@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -45,8 +45,8 @@ static int do_validate_object(const obj_header_t * ptr, const clump_t * cp,
 clump_t *
 gc_locate(const void *ptr, gc_state_t * gcst)
 {
-    const gs_ref_memory_t *mem;
-    const gs_ref_memory_t *other;
+    gs_ref_memory_t *mem;
+    gs_ref_memory_t *other;
 
     if (clump_locate(ptr, &gcst->loc))
         return gcst->loc.cp;
@@ -57,7 +57,7 @@ gc_locate(const void *ptr, gc_state_t * gcst)
      * is the stable one, the non-stable allocator of this space.
      */
 
-    if ((other = (const gs_ref_memory_t *)mem->stable_memory) != mem ||
+    if ((other = (gs_ref_memory_t *)mem->stable_memory) != mem ||
         (other = gcst->spaces_indexed[mem->space >> r_space_shift]) != mem
         ) {
         gcst->loc.memory = other;
@@ -155,7 +155,7 @@ gc_locate(const void *ptr, gc_state_t * gcst)
 
 /* Define the structure for temporarily saving allocator state. */
 typedef struct alloc_temp_save_s {
-        clump_t cc;
+        clump_t *cc;
         uint rsize;
         ref rlast;
 } alloc_temp_save_t;
@@ -163,34 +163,31 @@ typedef struct alloc_temp_save_s {
 static void
 alloc_temp_save(alloc_temp_save_t *pats, gs_ref_memory_t *mem)
 {
-    clump_t *pcc = mem->pcc;
-    obj_header_t *rcur = mem->cc.rcur;
+    obj_header_t *rcur;
 
-    if (pcc != 0) {
-        pats->cc = *pcc;
-        *pcc = mem->cc;
-    }
+    pats->cc = mem->cc;
+    if (mem->cc == NULL)
+        return;
+    rcur = mem->cc->rcur;
     if (rcur != 0) {
         pats->rsize = rcur[-1].o_size;
-        rcur[-1].o_size = mem->cc.rtop - (byte *) rcur;
+        rcur[-1].o_size = mem->cc->rtop - (byte *) rcur;
         /* Create the final ref, reserved for the GC. */
-        pats->rlast = ((ref *) mem->cc.rtop)[-1];
-        make_mark((ref *) mem->cc.rtop - 1);
+        pats->rlast = ((ref *) mem->cc->rtop)[-1];
+        make_mark((ref *) mem->cc->rtop - 1);
     }
 }
 /* Restore the temporarily saved state. */
 static void
 alloc_temp_restore(alloc_temp_save_t *pats, gs_ref_memory_t *mem)
 {
-    clump_t *pcc = mem->pcc;
-    obj_header_t *rcur = mem->cc.rcur;
+    obj_header_t *rcur;
 
-    if (rcur != 0) {
+    if (mem->cc && (rcur = mem->cc->rcur) != NULL) {
         rcur[-1].o_size = pats->rsize;
-        ((ref *) mem->cc.rtop)[-1] = pats->rlast;
+        ((ref *) mem->cc->rtop)[-1] = pats->rlast;
     }
-    if (pcc != 0)
-        *pcc = pats->cc;
+    mem->cc = pats->cc;
 }
 
 /* Validate the contents of an allocator. */
@@ -602,16 +599,23 @@ do_validate_object(const obj_header_t * ptr, const clump_t * cp,
     }
     if ((cp != 0 && !object_size_valid(pre, size, cp)) ||
         otype->ssize == 0 ||
-        size % otype->ssize != 0 ||
         (oname = struct_type_name_string(otype),
          *oname < 33 || *oname > 126)
         ) {
-        mlprintf2(gcst->heap, "Bad object 0x%lx(%lu),\n",
+        mlprintf2(gcst->heap, "\n Bad object 0x%lx(%lu),\n",
                   (ulong) ptr, (ulong) size);
         dmprintf2(gcst->heap, " ssize = %u, in clump 0x%lx!\n",
                   otype->ssize, (ulong) cp);
         return 1;
     }
+    if (size % otype->ssize != 0) {
+        mlprintf3(gcst->heap, "\n Potentially bad object 0x%lx(%lu), in clump 0x%lx!\n",
+                  (ulong) ptr, (ulong) size, (ulong) cp);
+        dmprintf3(gcst->heap, " structure name = %s, size = %u, ssize = %u\n",
+                  oname, size, otype->ssize);
+        dmprintf(gcst->heap, " This can happen (and is benign) if a device has been subclassed\n");
+    }
+
     return 0;
 }
 
