@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zdscpars.c 9043 2008-08-28 22:48:19Z giles $ */
+
 /* C language interface routines to DSC parser */
 
 /*
@@ -101,9 +103,13 @@ typedef struct dsc_data_s {
 } dsc_data_t;
 
 /* Structure descriptors */
-static void dsc_finalize(void *vptr);
+static void dsc_finalize(const gs_memory_t *cmem, void *vptr);
 gs_private_st_simple_final(st_dsc_data_t, dsc_data_t, "dsc_data_struct", dsc_finalize);
 
+static void *zDSC_memalloc (size_t size, void *closure_data);
+static void zDSC_memfree(void *ptr, void *closure_data);
+
+
 /* Define the key name for storing the instance pointer in a dictionary. */
 static const char * const dsc_dict_name = "DSC_struct";
 
@@ -115,11 +121,25 @@ static const char * const dsc_dict_name = "DSC_struct";
  */
 static int
 dsc_error_handler(void *caller_data, CDSC *dsc, unsigned int explanation,
-		  const char *line, unsigned int line_len)
+                  const char *line, unsigned int line_len)
 {
     return CDSC_OK;
 }
 
+static void *zDSC_memalloc (size_t size, void *closure_data)
+{
+    gs_memory_t *cmem = (gs_memory_t *)closure_data;
+
+    return(gs_alloc_bytes(cmem, size, "zDSC_memalloc: DSC parsing memory alloc"));
+}
+
+static void zDSC_memfree(void *ptr, void *closure_data)
+{
+    gs_memory_t *cmem = (gs_memory_t *)closure_data;
+
+    gs_free_object(cmem, ptr, "zDSC_memfree: DSC parsing memory free");
+}
+
 /*
  * This operator creates a new, initialized instance of the DSC parser.
  */
@@ -130,22 +150,32 @@ zinitialize_dsc_parser(i_ctx_t *i_ctx_p)
     ref local_ref;
     int code;
     os_ptr const op = osp;
-    dict * const pdict = op->value.pdict;
-    gs_memory_t * const mem = (gs_memory_t *)dict_memory(pdict);
-    dsc_data_t * const data =
-	gs_alloc_struct(mem, dsc_data_t, &st_dsc_data_t, "DSC parser init");
+    dict *pdict;
+    gs_memory_t *mem;
+    dsc_data_t *data;
 
+    if (ref_stack_count(&o_stack) < 1)
+        return_error(gs_error_stackunderflow);
+
+    check_read_type(*op, t_dictionary);
+
+    pdict = op->value.pdict;
+    mem = (gs_memory_t *)dict_memory(pdict);
+
+    data = gs_alloc_struct(mem, dsc_data_t, &st_dsc_data_t, "DSC parser init");
     if (!data)
-        return_error(e_VMerror);
+        return_error(gs_error_VMerror);
     data->document_level = 0;
-    data->dsc_data_ptr = dsc_init((void *) "Ghostscript DSC parsing");
+
+    data->dsc_data_ptr = dsc_init_with_alloc((void *) "Ghostscript DSC parsing",
+                           zDSC_memalloc, zDSC_memfree, (void *)mem->non_gc_memory);
     if (!data->dsc_data_ptr)
-    	return_error(e_VMerror);
+        return_error(gs_error_VMerror);
     dsc_set_error_function(data->dsc_data_ptr, dsc_error_handler);
     make_astruct(&local_ref, a_readonly | r_space(op), (byte *) data);
     code = idict_put_string(op, dsc_dict_name, &local_ref);
     if (code >= 0)
-	pop(1);
+        pop(1);
     return code;
 }
 
@@ -153,16 +183,16 @@ zinitialize_dsc_parser(i_ctx_t *i_ctx_p)
  * This routine will free the memory associated with Russell's parser.
  */
 static void
-dsc_finalize(void *vptr)
+dsc_finalize(const gs_memory_t *cmem, void *vptr)
 {
     dsc_data_t * const st = vptr;
+    (void)cmem; /* unused */
 
     if (st->dsc_data_ptr)
-	dsc_free(st->dsc_data_ptr);
+        dsc_free(st->dsc_data_ptr);
     st->dsc_data_ptr = NULL;
 }
 
-
 /* ---------------- Parsing ---------------- */
 
 /* ------ Utilities for returning values ------ */
@@ -177,7 +207,7 @@ dsc_put_int(gs_param_list *plist, const char *keyname, int value)
 /* Return a string value. */
 static int
 dsc_put_string(gs_param_list *plist, const char *keyname,
-	       const char *string)
+               const char *string)
 {
     gs_param_string str;
 
@@ -188,14 +218,14 @@ dsc_put_string(gs_param_list *plist, const char *keyname,
 /* Return a BoundingBox value. */
 static int
 dsc_put_bounding_box(gs_param_list *plist, const char *keyname,
-		     const CDSCBBOX *pbbox)
+                     const CDSCBBOX *pbbox)
 {
     /* pbbox is NULL iff the bounding box values was "(atend)". */
     int values[4];
     gs_param_int_array va;
 
     if (!pbbox)
-	return 0;
+        return 0;
     values[0] = pbbox->llx;
     values[1] = pbbox->lly;
     values[2] = pbbox->urx;
@@ -255,7 +285,7 @@ dsc_page(gs_param_list *plist, const CDSC *pData)
 
     if (page_num)		/* If we have page information */
         return dsc_put_int(plist, "PageNum",
-		       pData->page[page_num - 1].ordinal );
+                       pData->page[page_num - 1].ordinal );
     else			/* No page info - so return page=0 */
         return dsc_put_int(plist, "PageNum", 0 );
 }
@@ -298,18 +328,18 @@ dsc_page_orientation(gs_param_list *plist, const CDSC *pData)
      * will be in page_orientation.
      */
     if (page_num && pData->page[page_num - 1].orientation != CDSC_ORIENT_UNKNOWN)
-	return dsc_put_int(plist, "PageOrientation",
-			convert_orient(pData->page[page_num - 1].orientation));
+        return dsc_put_int(plist, "PageOrientation",
+                        convert_orient(pData->page[page_num - 1].orientation));
     else
         return dsc_put_int(plist, "Orientation",
-			   convert_orient(pData->page_orientation));
+                           convert_orient(pData->page_orientation));
 }
 
 static int
 dsc_orientation(gs_param_list *plist, const CDSC *pData)
 {
-    return dsc_put_int(plist, "Orientation", 
-			   convert_orient(pData->page_orientation));
+    return dsc_put_int(plist, "Orientation",
+                           convert_orient(pData->page_orientation));
 }
 
 static int
@@ -326,12 +356,13 @@ dsc_viewing_orientation(gs_param_list *plist, const CDSC *pData)
      * 'defaults' section or in a page section.
      */
     if (page_num && pData->page[page_num - 1].viewing_orientation != NULL) {
-	key = "PageViewingOrientation";
-	pctm = pData->page[page_num - 1].viewing_orientation;
-    } else {
+        key = "PageViewingOrientation";
+        pctm = pData->page[page_num - 1].viewing_orientation;
+    } else if (pData->viewing_orientation) {
         key = "ViewingOrientation";
-	pctm = pData->viewing_orientation;
-    }
+        pctm = pData->viewing_orientation;
+    } else
+    	return 0; /* ignore broken comment */
     values[0] = pctm->xx;
     values[1] = pctm->xy;
     values[2] = pctm->yx;
@@ -350,12 +381,12 @@ typedef struct cmdlist_s {
     int code;			/* Russell's DSC parser code (see dsc.h) */
     const char *comment_name;	/* A name to be returned to postscript caller */
     int (*dsc_proc) (gs_param_list *, const CDSC *);
-				/* A routine for transferring parameter values
-				   from C data structure to postscript dictionary
-				   key/value pairs. */
+                                /* A routine for transferring parameter values
+                                   from C data structure to postscript dictionary
+                                   key/value pairs. */
 } cmdlist_t;
 
-static const cmdlist_t DSCcmdlist[] = { 
+static const cmdlist_t DSCcmdlist[] = {
     { CDSC_PSADOBE,	    "Header",		dsc_adobe_header },
     { CDSC_CREATOR,	    "Creator",		dsc_creator },
     { CDSC_CREATIONDATE,    "CreationDate",	dsc_creation_date },
@@ -415,6 +446,7 @@ zparse_dsc_comments(i_ctx_t *i_ctx_p)
      * is bad, so ...).
      */
     check_type(*opString, t_string);
+    check_type(*opDict, t_dictionary);
     check_dict_write(*opDict);
     ssize = r_size(opString);
     if (ssize > MAX_DSC_MSG_SIZE)   /* need room for EOL + \0 */
@@ -437,13 +469,13 @@ zparse_dsc_comments(i_ctx_t *i_ctx_p)
      */
     while (*pBadList && strncmp(*pBadList, dsc_buffer, strlen(*pBadList)))
         pBadList++;
-    if (*pBadList) {		    /* If found in list, then skip comment */	
+    if (*pBadList) {		    /* If found in list, then skip comment */
         comment_code = 0;	    /* Ignore */
         if (dsc_buffer[2] == 'B') {
-	    dsc_state->document_level++;
-	} else if (dsc_state->document_level > 0) {
+            dsc_state->document_level++;
+        } else if (dsc_state->document_level > 0) {
             dsc_state->document_level--;
-	}
+        }
     }
     else if (dsc_state->document_level > 0) {
        comment_code = 0;	    /* Ignore */
@@ -454,30 +486,30 @@ zparse_dsc_comments(i_ctx_t *i_ctx_p)
          * location of the data area for the parser in our DSC dict.
          */
         comment_code = dsc_scan_data(dsc_state->dsc_data_ptr, dsc_buffer, ssize + 1);
-        if_debug1('%', "[%%].parse_dsc_comments: code = %d\n", comment_code);
-	/*
-	 * We ignore any errors from Russell's parser.  The only value that
-	 * it will return for an error is -1 so there is very little information.
-	 * We also do not want bad DSC comments to abort processing of an
-	 * otherwise valid PS file.
-	 */
+        if_debug1m('%', imemory, "[%%].parse_dsc_comments: code = %d\n", comment_code);
+        /*
+         * We ignore any errors from Russell's parser.  The only value that
+         * it will return for an error is -1 so there is very little information.
+         * We also do not want bad DSC comments to abort processing of an
+         * otherwise valid PS file.
+         */
         if (comment_code < 0)
-	    comment_code = 0;
+            comment_code = 0;
     }
     /*
      * Transfer data from DSC structure to postscript variables.
      * Look up proper handler in the local cmd decode list.
      */
     while (pCmdList->code && pCmdList->code != comment_code )
-	pCmdList++;
+        pCmdList++;
     if (pCmdList->dsc_proc) {
-	code = dict_param_list_write(&list, opDict, NULL, iimemory);
-	if (code < 0)
-	    return code;
-	code = (pCmdList->dsc_proc)((gs_param_list *)&list, dsc_state->dsc_data_ptr);
-	iparam_list_release(&list);
-	if (code < 0)
-	    return code;
+        code = dict_param_list_write(&list, opDict, NULL, iimemory);
+        if (code < 0)
+            return code;
+        code = (pCmdList->dsc_proc)((gs_param_list *)&list, dsc_state->dsc_data_ptr);
+        iparam_list_release(&list);
+        if (code < 0)
+            return code;
     }
 
     /* Put DSC comment name onto operand stack (replace string). */
