@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2018 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,12 +9,13 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
 /* Level 2 sethalftone operator */
+#include "memory_.h"
 #include "ghost.h"
 #include "oper.h"
 #include "gsstruct.h"
@@ -30,6 +31,8 @@
 #include "store.h"
 #include "iname.h"
 #include "zht2.h"
+#include "gxgstate.h"
+#include "gen_ordered.h"
 
 /* Forward references */
 static int dict_spot_params(const ref *, gs_spot_halftone *, ref *, ref *,
@@ -331,6 +334,7 @@ zsethalftone5(i_ctx_t *i_ctx_p)
                     break;
                 /* falls through */
             case ht_type_threshold:
+            case ht_type_threshold2:
                 if (!r_has_type(tprocs + j, t__invalid)) {
                     /* Schedule TransferFunction sampling. */
                     /****** check_xstack IS WRONG ******/
@@ -367,6 +371,183 @@ zsethalftone5(i_ctx_t *i_ctx_p)
     return (ref_stack_count(&e_stack) > edepth ? o_push_estack : 0);
 }
 
+/* <dict> .genordered <string> */
+/*         array will have: width height turn_on_sequence.x turn_on_sequence.y ...	*/
+/*         total array length is 2 + (2 * width * height)				*/
+static int
+zgenordered(i_ctx_t *i_ctx_p)
+{
+    os_ptr op = osp;
+    int i, code = 0;
+    gs_memory_t *mem;
+    int space_index;
+    htsc_param_t params;
+    int S;
+    htsc_dig_grid_t final_mask;
+    float tmp_float;
+    gs_gstate *pgs = igs;
+    gx_device *currdevice = pgs->device;
+    output_format_type output_type = OUTPUT_PS;
+    ref *out_type_name;
+
+    if (ref_stack_count(&o_stack) < 1)
+        return_error(gs_error_stackunderflow);
+    check_type(*op, t_dictionary);
+
+    space_index = r_space_index(op);		/* used to construct array that is returned */
+    mem = (gs_memory_t *) idmemory->spaces_indexed[space_index];
+
+    check_type(*op, t_dictionary);
+    check_dict_read(*op);
+
+    htsc_set_default_params(&params);
+    /* Modify the default HResolution and VResolution to be the device HWResolution */
+    params.horiz_dpi = currdevice->HWResolution[0];
+    params.vert_dpi = currdevice->HWResolution[1];
+    final_mask.memory = mem->non_gc_memory;
+    final_mask.data = NULL;
+
+    if ((code = dict_find_string(op, "OutputType", &out_type_name)) > 0) {
+        ref namestr;
+
+        if (!r_has_type(out_type_name, t_name))
+            return gs_error_typecheck;
+        name_string_ref(imemory, out_type_name, &namestr);
+        if (r_size(&namestr) == 8 && !memcmp(namestr.value.bytes, "TOSArray", 8))
+            output_type = OUTPUT_TOS;
+        else if (r_size(&namestr) == 5 && !memcmp(namestr.value.bytes, "Type3", 5))
+            output_type = OUTPUT_PS;
+        else if (r_size(&namestr) == 12 && !memcmp(namestr.value.bytes, "ThreshString", 12))
+            output_type = OUTPUT_RAW;
+        else
+            return gs_error_undefined;
+    }
+    if ((code = dict_int_param(op, "Angle", 0, 360, 0, &params.targ_scr_ang)) < 0)
+        return gs_error_undefined;
+    if ((code = dict_int_param(op, "Frequency", 1, 0x7fff, 75, &params.targ_lpi)) < 0)
+        return gs_error_undefined;
+    if ((code = dict_float_param(op, "HResolution", 300., &tmp_float)) < 0)
+        return gs_error_undefined;
+    if (code == 0)
+        params.horiz_dpi = tmp_float;
+    if ((code = dict_float_param(op, "VResolution", 300., &tmp_float)) < 0)
+        return gs_error_undefined;
+    if (code == 0)
+        params.vert_dpi = tmp_float;
+    if ((code = dict_int_param(op, "Levels", 1, 0x7fff, 256, &params.targ_quant)) < 0)
+        return gs_error_undefined;
+    if (code == 0)
+        params.targ_quant_spec = true;
+    if ((code = dict_int_param(op, "SuperCellSize", 1, 0x7fff, 1, &params.targ_size)) < 0)
+        return gs_error_undefined;
+    if (code == 0)
+        params.targ_size_spec = true;
+    if ((code = dict_int_param(op, "DotShape", 0, CUSTOM - 1, 0, (int *)(&params.spot_type))) < 0)
+        return gs_error_undefined;
+    if ((code = dict_bool_param(op, "Holladay", false, &params.holladay)) < 0)
+        return gs_error_undefined;
+
+    params.output_format = OUTPUT_TOS;		/* we want this format */
+    code = htsc_gen_ordered(params, &S, &final_mask);
+
+    if (code < 0)
+        goto done;
+
+    switch (output_type) {
+      case OUTPUT_TOS:
+        /* Now return the mask info in an array [ width height turn_on.x turn_on.y ... ] */
+        code = ialloc_ref_array((ref *)op, a_all, 2 + (2 * final_mask.width * final_mask.height), "gen_ordered");
+        if (code < 0)
+            goto done;
+        make_int(&(op->value.refs[0]), final_mask.width);
+        make_int(&(op->value.refs[1]), final_mask.height);
+        for (i=0; i < 2 * final_mask.width * final_mask.height; i++)
+            make_int(&(op->value.refs[i+2]), final_mask.data[i]);
+        break;
+      case OUTPUT_RAW:
+      case OUTPUT_PS:
+    /* Return a threshold array string first two bytes are width (high byte first),
+     * next two bytes are height, followed by the threshold array (one byte per cell)
+     * PostScript can easily form a Type 3 Halftone Thresholds string from this
+     * using "getinterval".
+     */
+        {
+            /* Make a threshold array from the turn_on_sequence */
+            int level;
+            int cur_pix = 0;
+            int width = final_mask.width;
+            int num_pix = width * final_mask.height;
+            double delta_value = 1.0 / (double)(num_pix);
+            double end_value, cur_value = 0.0;
+            byte *thresh;
+            ref rval, thresh_ref;
+
+            code = gs_error_VMerror;	/* in case allocation of thresh fails */
+            if (output_type == OUTPUT_RAW) {
+                if ((thresh = ialloc_string(4 + num_pix, "gen_ordered"))  == 0)
+                    goto done;
+                *thresh++ = width >> 8;
+                *thresh++ = width & 0xff;
+                *thresh++ = final_mask.height >> 8;
+                *thresh++ = final_mask.height & 0xff;
+            } else if ((thresh = ialloc_string(num_pix, "gen_ordered"))  == 0)
+                    goto done;
+            /* The following is adapted from thresh_remap with the default linear map */
+            for (level=0; level<256; level++) {
+                end_value = (float)(1+level) / 255.;
+                if (end_value > 255.0)
+                    end_value = 255.0;		/* clamp in case of rounding errors */
+                while (cur_value < (end_value - (delta_value * (1./256.))) ||
+                       (cur_pix + 1) == (num_pix / 2) ) {	/* force 50% gray level */
+                    thresh[final_mask.data[2*cur_pix] + (width*final_mask.data[2*cur_pix+1])] = 255 - level;
+                    cur_pix++;
+                    if (cur_pix >= num_pix)
+                        break;
+                    cur_value += delta_value;
+                }
+                if (cur_pix >= num_pix)
+                    break;
+            }
+            /* now fill any remaining cells */
+            for (; cur_pix < num_pix; cur_pix++) {
+                thresh[final_mask.data[2 * cur_pix] + (width*final_mask.data[2 * cur_pix + 1])] = 0;
+            }
+            if (output_type == OUTPUT_RAW) {
+                make_string(&thresh_ref, a_all | icurrent_space, 4 + num_pix, thresh-4);
+                *op = thresh_ref;
+                code = 0;
+            } else {
+                /* output_type == OUTPUT_PS */
+                /* Return a HalftoneType 3 dictionary */
+                code = dict_create(4, op);
+                if (code < 0)
+                    goto done;
+                make_string(&thresh_ref, a_all | icurrent_space, num_pix, thresh);
+                if ((code = idict_put_string(op, "Thresholds", &thresh_ref)) < 0)
+                    goto done;
+                make_int(&rval, final_mask.width);
+                if ((code = idict_put_string(op, "Width", &rval)) < 0)
+                    goto done;
+                make_int(&rval, final_mask.height);
+                if ((code = idict_put_string(op, "Height", &rval)) < 0)
+                    goto done;
+                make_int(&rval, 3);
+                if ((code = idict_put_string(op, "HalftoneType", &rval)) < 0)
+                    goto done;
+            }
+        }
+        break;
+      default:
+        return gs_error_undefined;
+    }
+
+done:
+    if (final_mask.data != NULL)
+        gs_free_object(mem->non_gc_memory, final_mask.data, ".genordered");
+
+    return (code < 0 ? gs_error_undefined : 0);
+}
+
 /* Install the halftone after sampling. */
 static int
 sethalftone_finish(i_ctx_t *i_ctx_p)
@@ -404,6 +585,7 @@ const op_def zht2_l2_op_defs[] =
 {
     op_def_begin_level2(),
     {"2.sethalftone5", zsethalftone5},
+    {"1.genordered", zgenordered},
                 /* Internal operators */
     {"0%sethalftone_finish", sethalftone_finish},
     op_def_end(0)
@@ -525,6 +707,8 @@ dict_threshold2_params(const ref * pdict, gs_threshold2_halftone * ptp,
     uint size;
     int cw2, ch2;
 
+    ptp->transfer = (code > 0 ? (gs_mapping_proc) 0 : gs_mapped_transfer);
+
     if (code < 0 ||
         (code = cw2 = dict_int_param(pdict, "Width2", 0, 0x7fff, 0,
                                      &ptp->width2)) < 0 ||
